#-*- R -*-

library(nlme)
options( width = 65, digits = 5 )
options( contrasts = c(unordered = "contr.helmert", ordered = "contr.poly") )

# Chapter 1    Linear Mixed-Effects Models: Basic Concepts and Examples
##########################################################################
# 1.1 A Simple Example of Random Effects.  travel is the y-variable,
# Rail is the factor
##########################################################################

Rail

# model 1, no fixed nor random effects
fm1Rail.lm <- lm( travel ~ 1, data = Rail ) 
fm1Rail.lm

# Model 2, Rail as a fixed effect, no intercept
fm2Rail.lm <- lm( travel ~ Rail - 1, data = Rail ) 
fm2Rail.lm

# Model 3, Rail as a random effect, no fixed effects, using the default REML
fm1Rail.lme <- lme(travel ~ 1, data = Rail, random = ~ 1 | Rail)
summary( fm1Rail.lme )

# Model 3 again but using ML
fm1Rail.lmeML <- update( fm1Rail.lme, method = "ML" )
summary( fm1Rail.lmeML )

plot( fm1Rail.lme )   # produces Figure 1.4
intervals( fm1Rail.lme )
anova( fm1Rail.lme )

##########################################################################
# 1.2 A Randomized Block Design
#  effort is the y-variable, Type of stool is the factor
#  Subject is a block (all stools applied to each subject)
##########################################################################
plot.design( ergoStool )   # produces Figure 1.6

# Set up contrasts for the first model.  Show these for the
# subject 1 only.  Type 1 is stool type 1 versus 2, Type 2 is 
# stool type 3 versus 1 and 2; and Type 3 is stool type 4 versus 
# the others
contrasts( ergoStool$Type )
ergoStool1 <- ergoStool[ ergoStool$Subject == "1", ]
model.matrix( effort ~ Type, ergoStool1 )   # X matrix for Subject 1

# fit the model using these contrasts, where the blocks (ie the
# subjects) are a random effect and the stool type is a fixed effect.
fm1Stool <-
  lme(effort ~ Type, data = ergoStool, random = ~ 1 | Subject)
summary( fm1Stool )
anova( fm1Stool )

# set up a different set of contrasts and repeat the analysis.
# This one is the common 0,1 dummy variables used in other packages.
# To get a fixed effect treatment mean, you must add the intercept 
# with the coefficient given for that fixed effect level.  The 
# intercept is the mean of the first fixed effect level.
options( contrasts = c( factor = "contr.treatment",
                        ordered = "contr.poly" ) )
contrasts( ergoStool$Type )

fm2Stool <-
  lme(effort ~ Type, data = ergoStool, random = ~ 1 | Subject)
summary( fm2Stool )
anova( fm2Stool 

# Redefine type, by removing the intercept. This is the overspecified model
# where each coefficient is the fixed effect treatment mean.
model.matrix( effort ~ Type - 1, ergoStool1 )
fm3Stool <-
 lme(effort ~ Type - 1, data = ergoStool, random = ~ 1 | Subject)
summary( fm3Stool )
anova( fm3Stool )

# get confidence intervals for the first model contracts
intervals( fm1Stool )

# get a plot of residuals vs predicted values for each subject.
# These are the subject specific fitted values and the white
# noise residuals (called the conditional predicted and
# residual values in some texts.
plot( fm1Stool,   # produces Figure 1.8
      form = resid(., type = "p") ~ fitted(.) | Subject,
      abline = 0 )

##########################################################################
# 1.3  Mixed-effects Models for Replicated, Blocked Designs
# score is the y-variable, Machine is the factor, and Worker is the random
# effect (ie the Block).  Each worker was measured three times on each
# machine, so this is a replicated block design.
##########################################################################

# interaction plot first
with(Machines, interaction.plot( Machine, Worker, score, las = 1))   # Figure 1.10

# model 1. Assume there is no interaction between Worker and Machine.
# Then, there are two random effects, the Machine (ie the block) and
# replicates within Machines which is the residual error.
fm1Machine <-
  lme( score ~ Machine, data = Machines, random = ~ 1 | Worker,method="ML" )
fm1Machine

# Model 2. The second model updates model 1 by adding a Machine by Worker interaction 
# (ie block by treatment interaction).  This is called Machine in Worker
# on the R output
fm2Machine <- update( fm1Machine, random = ~ 1 | Worker/Machine, method="ML" )
fm2Machine

# Check if there is more support for Model 1 or Model 2
anova( fm1Machine, fm2Machine )

# Model 3.  Since model 2 appeared better, update model 2, but
# drop the fixed effect. 
fm3Machine <- update( fm2Machine, score~1,method="ML")
summary( fm3Machine )

# compare models 2 and 3 to see if there is a Machine effect.
anova(fm2Machine, fm3Machine )

##########################################################################
# 1.4 An Analysis of Covariance Model
# the y variable is a tooth measure called distance. Factors are sex, 
# but only females are used here, age is a covariate, and 
# other factors are the subject
# Subject is random.  
##########################################################################

# select only the females from the data
names( Orthodont )
levels( Orthodont$Sex )
OrthoFem <- Orthodont[ Orthodont$Sex == "Female", ]

# Model 1.  A linear model of distance versus age. Since
# Orthodont is a grouped data object, lmList does this for each
# subject.
fm1OrthF.lis <- lmList( distance ~ age, data = OrthoFem )
coef( fm1OrthF.lis )
intervals( fm1OrthF.lis )
plot( intervals ( fm1OrthF.lis ) )   # produces Figure 1.12

# Model 2.  A update of model 1 that uses age scaled around
# the average age. 
fm2OrthF.lis <- update( fm1OrthF.lis, distance ~ I( age - 11 ) )
plot( intervals( fm2OrthF.lis ) )    # produces Figure 1.13

# Model 3.  A linear mixed model of distance versus age
# as model 2, but subject is introduced as a random effect
# rather than fitting an equation for each subject. The intercept
# is allowed to vary by subject.  Uses the default REML fit.
fm1OrthF <-
  lme( distance ~ age, data = OrthoFem, random = ~ 1 | Subject )
summary( fm1OrthF )

# Model 4.  Same as Model 3 but using ML fit.
fm1OrthFM <- update( fm1OrthF, method = "ML" )
summary( fm1OrthFM )

# Model 5.  Same as model 3, but the intercept and slope with
# age vary as random effects.  Default REML fit. 
fm2OrthF <- update( fm1OrthF, random = ~ age | Subject )
summary(fm2OrthF)

# compare Model 3 versus Model 5 to see if the additional
# random age coeffcient by subject is needed.  It is not.
anova( fm1OrthF, fm2OrthF )

# get more information on the selected model, Model 3.  First, 
# compare the random effects using REML to those using ML.
random.effects( fm1OrthF )
ranef( fm1OrthFM )

# get coefficients and other information about Model 3 for
# each subject.  Since the random coefficient for age was not
# needed, the age coefficient is the same for all subjects.  
coef( fm1OrthF )
plot( compareFits(coef(fm1OrthF), coef(fm1OrthFM)))   # Figure 1.15
plot( augPred(fm1OrthF), aspect = "xy", grid = TRUE )   # Figure 1.16

##########################################################################
# 1.5  Models for Nested Classification Factors
# the y-variable is pixel intensity from a CT scan,
# the subject is a Dog, and there are two sides scanned, R or L
# R or L are considered a nested factor by Pinero and Bates, although
# this could be considered a split plot if R is always R and L is always L.
# Measures are taken for each Dog over several days. Day is a fixed effect. 
##########################################################################

# Model 1.  Linear model of pixel versus day and day squared,
# with Dog as a random effect for the intercept and day 
# coefficients and side by dog interaction 
# is a random effect for the intercept coefficient, termed
# side in Dog on the output.
fm1Pixel <- lme( pixel ~ day + I(day^2), data = Pixel,
  random = list( Dog = ~ day, Side = ~ 1 ) )
intervals( fm1Pixel )
plot( augPred( fm1Pixel ) )   # produces Figure 1.18
VarCorr( fm1Pixel )
summary( fm1Pixel )

#Model 2. This updated model 1 to only have a random effects for 
# dog, not for side.
fm2Pixel <- update( fm1Pixel, random = ~ day | Dog)
summary(fm2Pixel)
plot(augPred(fm2Pixel))

# check if the random intercept by side was needed. If yes,
# then Model 1 should be better, and it is.  
anova( fm1Pixel, fm2Pixel )

# Model 3. Update Model 1 again, by including a dog 
# as a random effect on the intercept only, and an
# interaction term between Dog and Side as another random
# effect on the intercept.
fm3Pixel <- update( fm1Pixel, random = ~ 1 | Dog/Side )
summary(fm3Pixel)
plot(augPred(fm3Pixel))

# Compare Model 1 with Model 3. Model 1 is again better.
anova( fm1Pixel, fm3Pixel )

# Model 4.  Update Model 1, but including side as a fixed effect
# as well as a random dog by side interaction)
fm4Pixel <- update( fm1Pixel, pixel ~ day + I(day^2) + Side )
summary( fm4Pixel )
plot(augPred(fm4Pixel))

# cannot dompare Model 4 with Model 1, unless both are fitted using ML,
# since fixed effects have changed.  Get ML versions and then 
# test.  Model 1 is a better choice again.  
fm1PixelML <- lme( pixel ~ day + I(day^2), data = Pixel,
  random = list( Dog = ~ day, Side = ~ 1 ), method='ML' )
fm4PixelML <- update( fm1PixelML, pixel ~ day + I(day^2) + Side, method='ML' )
anova( fm4PixelML, fm1PixelML )

##########################################################################
# 1.6  A Split-Plot Experiment
# yield is the y variable.  Variety is a fixed effect assigned at the
# whole plot, and nitrogen is a fixed effect assigned at the split plot.
#  Block is a random effect.  There is no replications of variety within
# each block.  
##########################################################################

# Model 1.  First, make an ordered factor of nitro, include
# nitro, variety, and nitro by Variety as fixed effects.
# Random effects are block and block by Variety (ie the
# whole plot error.  
fm1Oats <- lme( yield ~ ordered(nitro) * Variety, data = Oats,
  random = ~ 1 | Block/Variety, method='ML' )
anova( fm1Oats )

# Model 2.  Update model 1 by dropping the nitro by Variety 
# interaction, since that was not significant. 
fm2Oats <- update( fm1Oats, yield ~ ordered(nitro) + Variety, method='ML' )
anova( fm2Oats )

# Model 3. Since Variety is not significant,Model 2 could be updated
# to drop Variety.
fm3Oats <- update( fm1Oats, yield ~ ordered( nitro ) )
summary( fm3Oats )

# Model 4.  Use nitro as a continuous variable instead in this model.
fm4Oats <-
  lme( yield ~ nitro, data = Oats, random = ~ 1 | Block/Variety, method='ML' )
summary( fm4Oats )
VarCorr( fm4Oats )
intervals( fm4Oats )
plot(augPred(fm4Oats), aspect = 2.5, layout = c(6, 3),
     between = list(x = c(0, 0, 0.5, 0, 0))) # produces Figure 1.21




