#############################################################################################
# Nonlinear models to fit height-diameter equations
# NLIN_SWO.r UBC R workshop                      
##############################################################################################

# read in data and create data frame 'swodata' and examine names and dimensions
#############################################################################################

swodata <- read.csv('C:\\workfile\\UBC_RWORK\\swodata.csv')
dim(swodata)
names(swodata)

# trno - tree number identifier
# dbh - tree diameter at breat height (cm)
# cl - tree crown length (m)
# dead - status identifier (1=live, 0=dead)
# ht - tree height (m)
# bal - basal area in larger trees (m^2/ha)
# ba - basal area/ha (m^2/ha)
# tph - number of trees per hectare
# spp - tree species identifier
# ccfl - crown competition factor in larger trees
# standid - stand number identifier
# plotid - plot number identifier
# nsnag - number of snags/ ha
 
## Reduce the number of digits shown to 3 so that print outs are more legible.
## Remove extraneous significance stars to improve formatting and printing

options(digits=3, show.signif.stars=FALSE)

names(swodata) 
dim(swodata)

swodata$ht2 <- swodata$ht-1.37               ##Tree height above dbh
attach(swodata)
length(unique(standid))

#To get a first impression of the data, we make a conditional scatter plot using the function xyplot() 
#Load all required packages 
library(lattice)
library(nls2)
library(nlme) 

xyplot(ht ~ dbh | spp, data=swodata, ylab="Height (m)", xlab= "dbh (cm)")

# Subset data to fit H-D models by species ==DF, Douglas fir and PM=Pacific Madrone
## Select only live trees (i.e., dead=0)
DF <- subset(swodata, spp=="DF" & dead==0)
#PM <- subset(swodata, spp=="PM", dead=0)

# only use live Douglas-firs
attach(DF)
dim(DF)

################################################################################ 
## Fitting nonlinear models - Model 1: Chapman-Richards 
## b0 is fixed, unknown asymptotic height, b1 is fixed, unknown steepness parameter, which is always 
## negative,and b3 is unknown curvature parameter that determines the rate of increase.  
## b0, b1, and b2 are species dependent coefficients
##################################################################################

fit.CR<-nls(ht2~b0*((1-exp(b1*dbh))^b2),data=DF,start=list(b0=64, b1=-.01, b2=0.92),
  trace=TRUE, nls.control(maxiter = 500))

###############################################
#Finding starting values  
# 1. Graphical exploration to select starting values
#    Write the model form and associted coefficients 
# Use function to write the model form and coefficients
##
expFct <- function(x, b0, b1, b2) {
 b0*((1-exp(b1*x))^b2)
 }

## Assess the different starting values and examine the curve 
##
plot(ht2 ~ dbh, data=DF, xlab="dbh (cm)", ylab="height (m)", ylim=c(0,80))
curve(expFct(x, b0=60, b1=-0.2, b2=5), add=TRUE)

##Examine 5 starting values at once
plot(ht2 ~ dbh, data=DF, xlab="dbh (cm)", ylab="height (m)", ylim=c(0,80))
curve(expFct(x, b0=50, b1=-0.1, b2=1), add=TRUE, lty=2)
curve(expFct(x, b0=55, b1=-0.01, b2=0.9), add=TRUE, lty=3)      #,ylab=("b0=55, b1=-0.01")
curve(expFct(x, b0=60, b1=-0.3, b2=0.6), add=TRUE, lty=4)
curve(expFct(x, b0=65, b1=-0.1, b2=1), add=TRUE, lty=1)
curve(expFct(x, b0=70, b1=-0.1, b2=1), add=TRUE, lty=5)

####
# 2. Grid search
#Need nls2 library

grid.CR <-expand.grid(list(b0=seq(35,61, by=2), b1=c(-0.01), b2=seq(-1,2, by=0.5)))

fit2.CR <-nls2(ht2~exp(b0+b1*(dbh^b2)),data=DF,start=grid.CR, algorithm="brute-force")
fit2.CR
summary(fit2.CR)


fit3.CR<-nls2(ht2~expFct(dbh, b0, b1, b2),data=DF,start=grid.CR, algorithm="brute-force")
summary(fit3.CR)

####################################################################	

###some useful attributes or values 		
# Use the deviance method to obtain the SSE 
deviance(fit.CR)

#use the logLik method to obtain the logarithm of the likelihood function 
logLik(fit.CR) 

#use the coef method to list the parameter estimates. 
coef(fit.CR)

# print out summary of fit
summary(fit.CR)

dVal <- c(10,50,100,150)
predict(fit.CR, data.frame(dbh=dVal))       

# Obtain summary of he fitted values from the model fit.CR fit  
summary(fitted(fit.CR))

## Create a plot of the original data superimposed with the C-R fitted curve
## Plot of the fitted regression curve is used to examine the mean structure 
## 

plot(ht2~dbh, data=DF, ylim=c(0,90), ylab="Height (m)", xlim=c(0,180), xlab="dbh (cm)")
dVal <-with(DF, seq(min(dbh), max(dbh), length.out=100))
lines(dVal, predict(fit.CR, newdata=data.frame(dbh=dVal)),col="red")
abline(h=1.3+coef(fit.CR)[1], lty=2, col="green")

#############################################################################################
# check assumptions of equal variance and normality of residuals
# Residual = (observed value) - (predicted value)  for the C-R model
# #############################################################################################


# testing for normality using the Shapiro-Wilk test
shapiro.test(residuals(fit.CR))		

op <- par(mfrow=c(1,3),pty="s")
## Residual plot (difference between the response value and the corresponding fitted values)
##Please note heteroscedacity - we will fix this problem in the NLME session
plot(fitted(fit.CR), residuals(fit.CR), xlab="Fitted Valaues", ylab="Residuals")
abline(a=0, b=0)

###Examine independence using lag plot, which is a plot of each residual versus the lag-one residual
plot(residuals(fit.CR), c(residuals(fit.CR)[-1], NA), xlab="Residuals", ylab="Lagged Residuals")

hist(residuals(fit.CR))			# draws a histogram
par(op)

##########
## Model 2: Ratkowsky's nonlinear model
fit.rat<-nls(ht2~exp(b0+(b1/(dbh+b2))),data=DF,
			start=list(b0=4.3, b1=-49, b2=13), trace=TRUE, nls.control(maxiter = 500))

# print out summary of fit
summary(fit.rat)		

#Model fits and summary 
deviance(fit.rat)
logLik(fit.rat) 
coef(fit.rat)
summary(fit.rat)

##Create a plot of the originl data super imposed with the Ratkowsky fitted curve
##
plot(ht2~dbh, data=DF, ylim=c(0,90), ylab="Height(m)", xlim=c(0,180), xlab="dbh (cm)")
dVal <-with(DF, seq(min(dbh), max(dbh), length.out=100))
lines(dVal, predict(fit.rat, newdata=data.frame(dbh=dVal)), col="red")
abline(h=1.3+exp(coef(fit.rat)[1]), lty=2, col="green")

#############################################################################################
# check assumptions of equal variance and normality of residuals
# Residual = (observed value) - (predicted value)  for the Ratkowsky's model
# #############################################################################################

yhat <- fitted(fit.rat)
resid <- residuals(fit.rat)

op <- par(mfrow=c(1,3),pty="s")
plot(yhat,resid)		            # residual plot
abline(h=0)
qqnorm(resid)		                # normality plot
qqline(resid, lty=2)
hist(resid)			                # draws a histogram
par(op)

##Compare AIC  - if more than 10 select one model over the other (see Burnham and Anderson's (2002)
AIC(fit.CR)
AIC(fit.rat)
###F-test
anova(fit.CR,fit.rat)

##Would other stand variable improve the fit of the model?
## Model 2 - with stand and positional variables
fit.CR2<-nls(ht2~(a0+a1*ccfl+a2*ba)*((1-exp(b1*dbh))^b2),data=DF,
			start=list(a0=47, a1=0.065, a2=0.13, b1=-.017, b2=0.97), 
      control=nls.control(maxiter=9000, tol=0.01, minFactor=1/1000))

#Model fits and summary 
summary(fit.CR2)

# testing for normality using the following tests:
shapiro.test(residuals(fit.CR2)		# Shapiro-Wilk


logLik(fit.CR2) 

plot(fitted(fit.CR2), residuals(fit.CR2), xlab="Fitted Valaues", ylab="Residuals")
abline(a=0, b=0)
	
###F-test
anova(fit.CR,fit.rat,fit.CR2)

## Profile likelihood - indicates whether the linear approximation is perfect 
plot(profile(fit.CR), conf=0.95) 

opar1 <- par(mfrow=c(3,3), mar=c(4,4,2,1), las=1)
plot(profile(fit.CR), absVal=FALSE)
par(opar1)

# confidence intervals for the tau
# extract the profiled versions
confint(fit.CR)


### Fitting Techniques and improving efficiency
## passing first and second derviatives will make fitting NLIN efficient
ht.dr <- deriv(~ b0*(1-exp(b1*x^b2)), 
    c("b0", "b1", "b2"),
    function(x, b0, b1, b2) {})

wbl.nls <- nls(ht2~ht.dr(dbh, b0, b1, b2),start=list(b0=24.5, b1=-.03, b2=1.1), data=DF)
# Let us inspect the model object wbl.nls
summary(wbl.nls)

# Extracted from Andrew's ice-braker 
# What is the underlying distribution of hts? We can learn more about them using the profile () function
opar <- par(mfrow=c(2,2), mar=c(4,4,2,1), las=1)
plot(fitted(wbl.nls), DF$ht2, xlab = "Fitted Values", ylab = "Observed Values")
abline(0, 1, col="red")
plot(fitted(wbl.nls), residuals(wbl.nls, type="pearson"), xlab = "Fitted Values", ylab = "Standardized Residuals")
abline(h=0, col="red")
par(opar)

## Examine the assumptions of normality on the underlying distribution of the parameter estimates
## Profile likelihood - indicates whether the linear approximation is perfect 
## for each parameter estimates or not
## Estimate the profile t function for each parameter and plot the values
opar1 <- par(mfrow=c(1,3), mar=c(4,4,2,1), las=1)
plot(profile(wbl.nls), conf=0.95)            
par(opar1)

## Obtain the original tau values instead of absolute values and examine
## if curvature exist or the linear approximation is perfect.  The graph indicates 
# that the linear approximation is not perfect in b0 component.  The linear approximation
# appears acceptable for b1 abd b2, as there was not curvature for these parameter estimates.
opar1 <- par(mfrow=c(3,3), mar=c(4,4,2,1), las=1)
plot(profile(wbl.nls), absVal=FALSE)
par(opar1)

my.t <- qt(0.975, summary(wbl.nls)$df[2])

# approximately the large-sample 95% confidence interval
coef(summary(wbl.nls))[,1:3] %*% matrix(c(1,-my.t,1,my.t), nrow=3)

# confidence intervals for the tau
# extract the profiled versions
confint(wbl.nls)

### Remedies for model violations 
### 1. Weighted leaset squares, 2. Variance modeling, and 3. Transformations
                   
nfem1 <- nls(ht2 ~ b0 *((1-exp(b1*dbh))^b2),
		data=DF, start=list(b0=24.5, b1=-.03, b2=1.1),
 		trace=TRUE, nls.control(maxiter = 500))

summary(nfem1)
 
 
## Remedies for model violation
# 1. weighted nonlinear regression

fit.CRW1<-nls(ht2~b0*((1-exp(b1*(dbh)))^b2),data=DF,start=list(b0=50, b1=-.01, b2=0.92),
weights=1/dbh,trace=TRUE, nls.control(maxiter = 500))

fit.CRW2<-nls(ht2~b0*((1-exp(b1*dbh))^b2),data=DF,start=list(b0=50, b1=-.01, b2=0.92),
weights=1/dbh^2, trace=TRUE, nls.control(maxiter = 500))

yhat <- fitted(fit.CRW1)
resid <- residuals(fit.CRW1)

op <- par(mfrow=c(1,3),pty="s")
plot(yhat,resid)		            # residual plot
abline(h=0)
qqnorm(resid)		# normality plot
qqline(resid, lty=2)
hist(resid)			# draws a histogram
par(op)

anova(fit.CRW1,fit.CRW2)

## Remedies for model violation
### 2. Variance modeling
# Need nlme library for gnls

fit.CRWF <- gnls(ht2~b0*((1-exp(b1*dbh))^b2),data=DF,start=list(b0= 43.9, b1= -.02, b2= 1.02), 
 weights=varFixed(~dbh),verbose=TRUE, control=gnlsControl(nlsMaxIter=20,nlsTol = 0.01))

summary(fit.CRWF)

fit.CRWP <- gnls(ht2~b0*((1-exp(b1*dbh))^b2),data=DF,start=list(b0=50, b1=-.01, b2=0.92), 
weights=varPower(),verbose=TRUE, control=gnlsControl(nlsMaxIter=20,nlsTol = 0.01))

yhat <- fitted(fit.CRWF)
resid <- residuals(fit.CRWF)

op <- par(mfrow=c(1,3),pty="s")
plot(yhat,resid)		            # residual plot
abline(h=0)
qqnorm(resid)		                # normality plot
qqline(resid, lty=2)
hist(resid)			                # draws a histogram
par(op)

###More options in variance modeling
nfem1.w <- gnls(ht2 ~ b0 *((1-exp(b1*dbh))^b2),
		data=DF, start=list(b0= 43.9, b1= -.02, b2= 1.02), params = b0+b1+b2~1, weights=varPower(form=~dbh),
		verbose=TRUE, control=gnlsControl(nlsMaxIter=20,nlsTol = 0.01))

summary(nfem1.w)


##Compare weighed and unweighted models - need to use the same procedures for both
nfem1.now <- gnls(ht2 ~ b0 *((1-exp(b1*dbh))^b2),
		data=DF, start=list(b0= 43.9, b1= -.02, b2= 1.02), params = b0+b1+b2~1,  
		verbose=TRUE, control=gnlsControl(nlsMaxIter=20,nlsTol = 0.01))

plot(nfem1.now)
summary(nfem1.now)

anova(nfem1.w, nfem1.now)


## Remedies for model violation
### 3. Transformations
#Load nlme library
library(nlrwr)        #needed for cox-box 

##Transformation  using the Box-Cox family
fit.CRtr <- boxcox.nls(fit.CR)

bcSummary(fit.CRtr)

coef(summary(fit.CR))

coef(summary(fit.CRtr))

#str(fit.CRtr)

yhat <- fitted(fit.CRtr)
resid <- residuals(fit.CRtr)

# diagnostic plots
op <- par(mfrow=c(1,3),pty="s")
plot(yhat,resid)		            # residual plot
abline(h=0)
qqnorm(resid)		# normality plot
qqline(resid, lty=2)
hist(resid)			# draws a histogram
par(op)

##4. Sandwich estimators
library(sandwich)
vcov(fit.CR)           #variance -covariance matrix

sandwich(fit.CR)

##Obtain the estimated standard errors based on the sandwich estimator
coeftest(fit.CR, vcov=sandwich)

coeftest(fit.CR)
#Adjustment lead to higher SE - did not work


