#############################################################################################
# Nonlinear mixed models
#############################################################################################

# load necessary packages
library(nlme)


#############################################################################################

# read in data set 
setwd("C:\\temp\\course materials\\Day 2\\No. 4\\")
swodata <- read.csv('swodata.csv')
dim(swodata)
names(swodata)


# trno - tree number identifier
# dbh - tree diameter at breat height (cm)
# cl - tree crown length (m)
# dead - status identifier (1=live, 0=dead)
# ht - tree height (m)
# bal - basal area in larger trees (m^2/ha)
# ba - basal area/ha (m^2/ha)
# tph - number of trees per hectare
# spp - tree species identifier
# ccfl - crown competition factor in larger trees
# standid - stand number identifier
# plotid - plot number identifier
# nsnag - number of snags/ ha


# create new height variable by subtracting breast height: ht2 <- ht-1.37
swodata$ht2 <- swodata$ht-1.37

# only use live Douglas-firs
DF <- subset(swodata, spp=="DF" & dead==0,)
dim(DF)


#############################################################################################
#############################################################################################
# Section 1
#############################################################################################
#############################################################################################


#############################################################################################
# Nonlinear fixed effects model -- Basic model
#############################################################################################

##### 1a)  #####
nfem1 <- nls(ht2 ~ b0 *((1-exp(b1*dbh))^b2),
		data=DF, start=list(b0=24.5, b1=-.03, b2=1.1),
 		trace=TRUE, nls.control(maxiter = 500))
summary(nfem1)


##### 1b)  #####
# can use standard plotting commands, e.g.:
plot(nfem1)
qqnorm(nfem1)

# I prefer creating my own set of plots!
# extract fitted values and residuals	
fitted <- fitted(nfem1)				# fitted values
res.raw <- resid(nfem1)				# raw residuals
res.std <- resid(nfem1,type="pearson")	# standardized residuals

# diagnostic plots
op <- par(mfrow=c(2,2))
plot(fitted,DF$ht2, xlab="Fitted values", ylab="Observed values",
		main="Fitted vs. Observed"); 	abline(0,1,col="green")
plot(fitted,res.std, xlab="Fitted values", ylab="Standardized residuals",
		main="Residual plot"); abline(h=c(-2,0,2),col="green")
qqnorm(res.std); qqline(res.std)
par(op)


##### 1c)  #####
# weighted nonlinear regression model to meet assumptions
nfem1.w <- gnls(ht2 ~ b0 *((1-exp(b1*dbh))^b2),
		data=DF, start=list(b0=24.5, b1=-.03, b2=1.1), 
		weights=varPower(form=~dbh), verbose=TRUE)
summary(nfem1.w)

# extract fitted values and residuals	
fitted <- fitted(nfem1.w)			# fitted values
res.std <- resid(nfem1.w,type="pearson")	# standardized residuals
	
# diagnostic plots
op <- par(mfrow=c(2,2))
plot(fitted,DF$ht2, xlab="Fitted values", ylab="Observed values",
		main="Fitted vs. Observed"); 	abline(0,1,col="green")
plot(fitted,res.std, xlab="Fitted values", ylab="Standardized residuals",
		main="Residual plot"); abline(h=c(-2,0,2),col="green")
qqnorm(res.std); qqline(res.std)
par(op)


##### 1d)  #####
# compare the two models
# In order to use a likelihood ratio test, you have to refit the basic model with gnls()
nfem1 <- gnls(ht2 ~ b0 *((1-exp(b1*dbh))^b2),
		data=DF, start=list(b0= 43.9, b1= -.02, b2= 1.02), params = b0+b1+b2~1)

# likelihood ratio test
anova(nfem1,nfem1.w)


#############################################################################################
# Nonlinear fixed effects model -- 
#	asymptotic parameter dependent on stand-level variables ccfl and ba
#############################################################################################

##### 1e)  #####
nfem2 <- gnls(ht2 ~ (b00+b01*ccfl+b02*ba) *((1-exp(b1*dbh))^b2),
		data=DF, start=c(b00=36.2,  b01=0.036, b02= 0.25, b1=-0.018, b2=0.88))
summary(nfem2)


##### 1f)  #####
# extract fitted values and residuals	
fitted <- fitted(nfem2)			# fitted values
res.std <- resid(nfem2,type="pearson")	# standardized residuals

# diagnostic plots
op <- par(mfrow=c(2,2))
plot(fitted,DF$ht2, xlab="Fitted values", ylab="Observed values",
		main="Fitted vs. Observed"); 	abline(0,1,col="green")
plot(fitted,res.std, xlab="Fitted values", ylab="Standardized residuals",
		main="Residual plot"); abline(h=c(-2,0,2),col="green")

qqnorm(res.std); qqline(res.std)
par(op)


# weighted nonlinear regression model to meet assumptions
nfem2.w <- gnls(ht2 ~ (b00+b01*ccfl+b02*ba) *((1-exp(b1*dbh))^b2),
		data=DF, start=c(b00=36.2,  b01=0.036, b02= 0.25, b1=-0.018, b2=0.88),
		weights=varPower(form=~dbh))
summary(nfem2.w)


# extract fitted values and residuals	
fitted <- fitted(nfem2.w)				# fitted values
res.std <- resid(nfem2.w,type="pearson")	# standardized residuals

# diagnostic plots
op <- par(mfrow=c(2,2))
plot(fitted,DF$ht2, xlab="Fitted values", ylab="Observed values",
		main="Fitted vs. Observed"); 	abline(0,1,col="green")
plot(fitted,res.std, xlab="Fitted values", ylab="Standardized residuals",
		main="Residual plot"); abline(h=c(-2,0,2),col="green")

qqnorm(res.std); qqline(res.std)
par(op)

##### 1g)  #####
# likelihood ratio test
anova(nfem1.w,nfem2.w)




#############################################################################################
#############################################################################################
# Section 2
#############################################################################################
#############################################################################################


#############################################################################################
# Nonlinear mixed effects model -- one random stand effect
#############################################################################################

##### 2a)  #####
# boxplot of residuals by stand for nfem2.w
plot(nfem2.w, factor(standid)~resid(.), abline=0)

# if residuals mostly negative for some stands and mostly positive for others
# --> include 'standid effect' in model

##### 2b)  #####
# add random stand effect to model
nmem1 <-nlme(ht2~(b00+b01*ccfl+b02*ba)*((1-exp(b1*dbh))^b2),
			fixed = b00 + b01 + b02 + b1 + b2 ~ 1, random = b00 ~ 1 | standid,
			data=DF,start=c(b00=36.2,  b01=0.036, b02= 0.25, b1=-0.018, b2=0.88))
summary(nmem1)


##### 2c)  #####
# extract fitted values and residuals	
fitted <- fitted(nmem1)				# fitted values
res.std <- resid(nmem1,type="pearson")	# standardized residuals

# diagnostic plots
op <- par(mfrow=c(2,2))
plot(fitted,DF$ht2, xlab="Fitted values", ylab="Observed values",
		main="Fitted vs. Observed"); 	abline(0,1,col="green")
plot(fitted,res.std, xlab="Fitted values", ylab="Standardized residuals",
		main="Residual plot"); abline(h=c(-2,0,2),col="green")

qqnorm(res.std); qqline(res.std)
par(op)


##### 2d)  #####
# include weights to adjust for unequal variances

# use groupedData because it is needed for a plotting statement coming up
DF.grouped <- groupedData(ht2~dbh|standid, data=DF)  

nmem1.w <-nlme(ht2~(b00+b01*ccfl+b02*ba)*((1-exp(b1*dbh))^b2),
			fixed = b00 + b01 + b02 + b1 + b2 ~ 1, random = b00 ~ 1 | standid,
			data=DF.grouped,start=c(b00=36.2,
			b01=0.036, b02= 0.25, b1=-0.018, b2=0.88), weights=varPower(form=~dbh))
summary(nmem1.w)


# extract fitted values and residuals	
fitted <- fitted(nmem1.w)				# fitted values
res.std <- resid(nmem1.w,type="pearson")		# standardized residuals

# diagnostic plots
op <- par(mfrow=c(2,2))
plot(fitted,DF$ht2, xlab="Fitted values", ylab="Observed values",
		main="Fitted vs. Observed"); 	abline(0,1,col="green")
plot(fitted,res.std, xlab="Fitted values", ylab="Standardized residuals",
		main="Residual plot"); abline(h=c(-2,0,2),col="green")
qqnorm(res.std); qqline(res.std)
par(op)


##### 2e)  #####
# boxplots of residuals by stand
plot(nmem1, factor(standid)~resid(.), abline=0)

##### 2f)  #####
# plot of augmented predictions
plot(augPred(nmem1.w, level=0:1),layout=c(10,5))

##### 2g)  #####
# likelihood ratio test
anova(nfem2.w,nmem1.w)



#############################################################################################
#############################################################################################
# Section 3
#############################################################################################
#############################################################################################


#############################################################################################
# Nonlinear mixed effects model -- two random stand effects
#############################################################################################

##### 3a)  #####
# weighted model
nmem2.w <-nlme(ht2~(b00+b01*ccfl+b02*ba)*((1-exp(b1*dbh))^b2),
			fixed = b00 + b01 + b02 + b1 + b2 ~ 1, random = b00 + b2 ~ 1 | standid,
			data=DF.grouped,start=c(b00=36.2,  b01=0.036,
			b02= 0.25, b1=-0.018, b2=0.88), weights=varPower(form=~dbh))
summary(nmem2.w)

##### 3b)  #####
# extract fitted values and residuals	
fitted <- fitted(nmem2.w)				# fitted values
res.std <- resid(nmem2.w,type="pearson")	# standardized residuals

# diagnostic plots
op <- par(mfrow=c(2,2))
plot(fitted,DF$ht2, xlab="Fitted values", ylab="Observed values",
		main="Fitted vs. Observed"); 	abline(0,1,col="green")
plot(fitted,res.std, xlab="Fitted values", ylab="Standardized residuals",
		main="Residual plot"); abline(h=c(-2,0,2),col="green")

qqnorm(res.std); qqline(res.std)
par(op)


##### 3c)  #####
# test for normality of random effects
qqnorm(nmem2.w, ~ranef(.))

# I prefer my own plots
op <- par(mfrow=c(1,2), pty="s")
qqnorm(ranef(nmem2.w)[,1]); qqline(ranef(nmem2.w)[,1])
qqnorm(ranef(nmem2.w)[,2]); qqline(ranef(nmem2.w)[,2])
par(op)


##### 3d)  #####
# scatterplot of random effects
pairs(nmem2.w)

##### 3e)  #####
# boxplots of residuals by stand
plot(nmem2.w, factor(standid)~resid(.), abline=0)

##### 3f)  #####
# plot of augmented predictions
plot(augPred(nmem2.w, level=0:1),layout=c(10,5))

##### 3g)  #####
# likelihood ratio test
anova(nmem1.w, nmem2.w)